/*
 * Copyright (c) 2012-2014 Colin Harrison All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE ABOVE LISTED COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name(s) of the above copyright
 * holders shall not be used in advertising or otherwise to promote the sale,
 * use or other dealings in this Software without prior written authorization.
 *
 * Author: Colin Harrison
 */

 /* DesktopNolock: Stops the screensaver from locking the desktop */

#include <windows.h>
#include <psapi.h>

#define USAGE1 "DesktopNolock prevents the screensaver from locking the desktop.\n" \
               "It inputs zero mouse movement every 20 seconds.\n\n" \
               "Note: the executable must not be renamed.\n\n"

#define USAGE2 "Press OK to run process DesktopNolock in the background\n" \
               "or Cancel to not do this.\n\n" \
               "The process will toggle off when DesktopNolock is run again."

static int
WasRunning(DWORD processID)
{
    int proglen = 13;
    char *program = "DesktopNolock";
    char szProcessName[MAX_PATH] = "<unknown>";

    HANDLE hProcess = OpenProcess(PROCESS_QUERY_INFORMATION |
                                  PROCESS_VM_READ | PROCESS_TERMINATE,
                                  FALSE, processID);

    if (NULL != hProcess) {
        HMODULE hMod;
        DWORD cbNeeded;

        if (EnumProcessModules(hProcess, &hMod, sizeof(hMod), &cbNeeded))
            GetModuleBaseName(hProcess, hMod, szProcessName,
                              sizeof(szProcessName));

        if (!strncmp(szProcessName, program, proglen) &&
            GetCurrentProcessId() != processID) {
            TerminateProcess(hProcess, 0);
            CloseHandle(hProcess);
            return 1;
        }
    }

    CloseHandle(hProcess);
    return 0;
}

int
main(int argc, char *argv[])
{
    DWORD aProcesses[1024], cbNeeded, cProcesses;
    unsigned int i;
    INPUT imouse;

    if (argv[1]) {
        MessageBox(NULL, USAGE1, "DesktopNolock help",
                   MB_OK | MB_ICONEXCLAMATION);
        return 1;
    }

    if (!EnumProcesses(aProcesses, sizeof(aProcesses), &cbNeeded))
        return 1;

    cProcesses = cbNeeded / sizeof(DWORD);

    for (i = 0; i < cProcesses; i++) {
        if (aProcesses[i] != 0) {
            if (WasRunning(aProcesses[i])) {
                MessageBox(NULL, "Toggled off", "DesktopNolock",
                           MB_OK | MB_ICONINFORMATION);
                return 0;
            }
        }
    }

    if (MessageBox
        (NULL, USAGE1 USAGE2, "DesktopNolock",
         MB_OKCANCEL | MB_ICONINFORMATION) == 2)
        return 0;

    while (1) {
        imouse.type = INPUT_MOUSE;
        imouse.mi.dx = 0;
        imouse.mi.dy = 0;
        imouse.mi.mouseData = 0;
        imouse.mi.dwFlags = (MOUSEEVENTF_MOVE || MOUSEEVENTF_ABSOLUTE);
        imouse.mi.time = 0;
        imouse.mi.dwExtraInfo = GetMessageExtraInfo();
        SendInput(1, &imouse, sizeof(imouse));
        Sleep(20000);
    }
    return 0;
}
